@@ -271,7 +271,7 @@ class FrameBuffer : public Object
     using _super = Object;
 
 public:
-    static inline std::string ObjectName(void){ return "Tensor"; }
+    static inline std::string ObjectName(void){ return "FrameBuffer"; }
     std::string GetObjectName(void) const override { return ObjectName(); }
 
 protected:
@@ -669,7 +669,7 @@ public:
     {
         index_t auto_index = -1;
         index_t total = 1;
-        for (index_t i = (index_t)shape.size()-1; i >= 0;--i) {
+        for (std::intptr_t i = (std::intptr_t)shape.size()-1; i >= 0;--i) {
             if (shape[i] < 0) {
                 auto_index = i;
             }
@@ -678,7 +678,7 @@ public:
             }
         }
         if (auto_index >= 0) {
-            shape[auto_index] = m_node_size / total;
+            shape[(std::size_t)auto_index] = m_node_size / total;
         }
 
         // 再計算
@@ -713,9 +713,10 @@ public:
         m_tensor.FillZero();
     }
 
-//  void FillZeroMargin(void)
-//  {
-//  }
+    void Fill(double value)
+    {
+        m_tensor = value;
+    }
     
 
     int     GetType(void)  const { return m_data_type; }
@@ -853,8 +854,8 @@ protected:
             bbcu_ConvBitToReal<float>(
                     (int const *)src_ptr.GetAddr(),
                     (float     *)dst_ptr.GetAddr(),
-                    0.0f,
-                    1.0f,
+                    -1.0f,
+                    +1.0f,
                     (int)this->GetNodeSize(),
                     (int)this->GetFrameSize(),
                     (int)(this->GetFrameStride() / sizeof(int)),
@@ -943,7 +944,7 @@ protected:
 
         index_t stride = 1;
         index_t index = 0;
-        for ( index_t i = (index_t)m_node_shape.size()-1; i>= 0; --i ) {
+        for ( std::intptr_t i = (std::intptr_t)m_node_shape.size()-1; i>= 0; --i ) {
             BB_DEBUG_ASSERT(indices[i] >= 0 && indices[i] < m_node_shape[i]);
             index += stride * indices[i];
             stride *= m_node_shape[i];
@@ -1232,9 +1233,9 @@ public:
 
         auto ptr = Lock<Tp>();
         for (index_t frame = 0; frame < m_frame_size; ++frame) {
-            BB_ASSERT(data[frame].size() == (size_t)m_node_size);
+            BB_ASSERT(data[(size_t)frame].size() == (size_t)m_node_size);
             for (index_t node = 0; node < m_node_size; ++node) {
-                ptr.Set(frame, node, data[frame + offset][node]);
+                ptr.Set(frame, node, data[(size_t)(frame + offset)][(size_t)node]);
             }
         }
     }
@@ -1396,7 +1397,7 @@ public:
             #pragma omp parallel for
             for (index_t node = 0; node < m_node_size; ++node)
             {
-                memcpy(dst_addr + buf.m_frame_stride * node, src_addr + m_frame_stride * node + byte_offset, byte_size);
+                memcpy(dst_addr + buf.m_frame_stride * node, src_addr + m_frame_stride * node + byte_offset, (std::size_t)byte_size);
             }
         }
 
@@ -1417,7 +1418,7 @@ public:
         auto dst_addr = (std::int8_t       *)dst_ptr.GetAddr();
 
         index_t stride_size = GetFrameStride();
-        memcpy(dst_addr, src_addr + (m_frame_stride * offset), stride_size * size);
+        memcpy(dst_addr, src_addr + (m_frame_stride * offset), (std::size_t)(stride_size * size));
 
         return buf;
     }
@@ -1437,14 +1438,14 @@ public:
         {
             auto src0_ptr  = m_tensor.LockMemoryConst();
             auto src0_addr = (std::int8_t const *)src0_ptr.GetAddr();
-            memcpy(dst_addr, src0_addr, m_frame_stride * m_node_size);
+            memcpy(dst_addr, src0_addr, (std::size_t)(m_frame_stride * m_node_size));
             dst_addr += m_frame_stride * m_node_size;
         }
 
         {
             auto src1_ptr  = buf.m_tensor.LockMemoryConst();
             auto src1_addr = (std::int8_t const *)src1_ptr.GetAddr();
-            memcpy(dst_addr, src1_addr, buf.m_frame_stride * buf.m_node_size);
+            memcpy(dst_addr, src1_addr, (std::size_t)(buf.m_frame_stride * buf.m_node_size));
         }
 
         return dst_buf;
@@ -1456,6 +1457,130 @@ public:
     //  演算
     // -------------------------------------
 
+protected:
+    template<typename T> 
+    bool _IsNan(void)
+    {
+#ifdef BB_WITH_CUDA
+        if ( this->IsDeviceAvailable() && Manager::IsDeviceAvailable() ) {
+            Tensor_<int> result(1);
+            {
+                auto ptr_buf    = this->LockDeviceMemoryConst();
+                auto ptr_result = result.LockDeviceMemory(true);
+                bbcu_FrameBuf_IsnNan
+                    (
+                        (int        *)ptr_result.GetAddr(),
+                        (T   const  *)ptr_buf.GetAddr(),
+                        (int         )this->GetNodeSize(),
+                        (int         )this->GetFrameSize(),
+                        (int         )(this->GetFrameStride() / sizeof(T))
+                    );
+            }
+            {
+                auto dst_ptr = result.LockConst();
+                return dst_ptr(0) != 0;
+            }
+        }
+#endif
+
+        auto ptr = LockConst<T>();
+        for ( index_t node = 0; node < GetNodeSize(); ++node ) {
+            for ( index_t frame = 0; frame < GetFrameSize(); ++frame ) {
+                if ( std::isnan(ptr.Get(frame, node)) ) {
+                    return true;
+                }
+            }
+        }
+        return false;
+    }
+
+
+    template<typename T> 
+    double _Min(void)
+    {
+        T   value = std::numeric_limits<T>::max();
+        auto ptr = LockConst<T>();
+        for ( index_t node = 0; node < GetNodeSize(); ++node ) {
+            for ( index_t frame = 0; frame < GetFrameSize(); ++frame ) {
+                value = std::min(value, ptr.Get(frame, node));
+            }
+        }
+        return (double)value;
+    }
+
+
+    template<typename T> 
+    double _MinFp(void)
+    {
+#ifdef BB_WITH_CUDA
+        if ( this->IsDeviceAvailable() && Manager::IsDeviceAvailable() ) {
+            Tensor_<T> result(1);
+            {
+                auto ptr_buf    = this->LockDeviceMemoryConst();
+                auto ptr_result = result.LockDeviceMemory(true);
+                bbcu_FrameBuf_Min<T>
+                    (
+                        (T          *)ptr_result.GetAddr(),
+                        (T   const  *)ptr_buf.GetAddr(),
+                        (int         )this->GetNodeSize(),
+                        (int         )this->GetFrameSize(),
+                        (int         )(this->GetFrameStride() / sizeof(T))
+                    );
+            }
+            {
+                auto dst_ptr = result.LockConst();
+                return (double)dst_ptr(0);
+            }
+        }
+#endif
+        return _Min<T>();
+    }
+
+    
+    template<typename T> 
+    double _Max(void)
+    {
+        T   value = std::numeric_limits<T>::lowest();
+        auto ptr = LockConst<T>();
+        for ( index_t node = 0; node < GetNodeSize(); ++node ) {
+            for ( index_t frame = 0; frame < GetFrameSize(); ++frame ) {
+                value = std::max(value, ptr.Get(frame, node));
+            }
+        }
+        return (double)value;
+    }
+
+
+    template<typename T> 
+    double _MaxFp(void)
+    {
+#ifdef BB_WITH_CUDA
+        if ( this->IsDeviceAvailable() && Manager::IsDeviceAvailable() ) {
+            Tensor_<T> result(1);
+            {
+                auto ptr_buf    = this->LockDeviceMemoryConst();
+                auto ptr_result = result.LockDeviceMemory(true);
+                bbcu_FrameBuf_Max<T>
+                    (
+                        (T          *)ptr_result.GetAddr(),
+                        (T   const  *)ptr_buf.GetAddr(),
+                        (int         )this->GetNodeSize(),
+                        (int         )this->GetFrameSize(),
+                        (int         )(this->GetFrameStride() / sizeof(T))
+                    );
+            }
+            {
+                auto dst_ptr = result.LockConst();
+                return (double)dst_ptr(0);
+            }
+        }
+#endif
+        return _Max<T>();
+    }
+
+
+public:
+    inline FrameBuffer& operator=(double src)       { m_tensor  = src; return *this; }
     inline FrameBuffer& operator+=(FrameBuffer src) { m_tensor += src.m_tensor; return *this; }
     inline FrameBuffer& operator+=(double src)      { m_tensor += src; return *this; }
     inline FrameBuffer& operator-=(FrameBuffer src) { m_tensor -= src.m_tensor; return *this; }
@@ -1465,6 +1590,52 @@ public:
     inline FrameBuffer& operator/=(FrameBuffer src) { m_tensor /= src.m_tensor; return *this; }
     inline FrameBuffer& operator/=(double src)      { m_tensor /= src; return *this; }
 
+    bool IsNan(void)
+    {
+        switch ( GetType() ) {
+        case BB_TYPE_FP32: return _IsNan<float>();
+        case BB_TYPE_FP64: return _IsNan<double>();
+        }
+        return false;
+    }
+    
+    double Min(void)
+    {
+        switch ( GetType() ) {
+        case BB_TYPE_FP32:   return _MinFp<float >();
+        case BB_TYPE_FP64:   return _MinFp<double>();
+        case BB_TYPE_BIT:    return _Min<Bit     >();
+        case BB_TYPE_INT8:   return _Min<int8_t  >();
+        case BB_TYPE_INT16:  return _Min<int16_t >();
+        case BB_TYPE_INT32:  return _Min<int32_t >();
+        case BB_TYPE_INT64:  return _Min<int64_t >();
+        case BB_TYPE_UINT8:  return _Min<uint8_t >();
+        case BB_TYPE_UINT16: return _Min<uint16_t>();
+        case BB_TYPE_UINT32: return _Min<uint32_t>();
+        case BB_TYPE_UINT64: return _Min<uint64_t>();
+        default:   BB_ASSERT(0);
+        }
+        return 0;
+    }
+    
+    double Max(void)
+    {
+        switch ( GetType() ) {
+        case BB_TYPE_FP32:   return _MaxFp<float >();
+        case BB_TYPE_FP64:   return _MaxFp<double>();
+        case BB_TYPE_BIT:    return _Max<Bit     >();
+        case BB_TYPE_INT8:   return _Max<int8_t  >();
+        case BB_TYPE_INT16:  return _Max<int16_t >();
+        case BB_TYPE_INT32:  return _Max<int32_t >();
+        case BB_TYPE_INT64:  return _Max<int64_t >();
+        case BB_TYPE_UINT8:  return _Max<uint8_t >();
+        case BB_TYPE_UINT16: return _Max<uint16_t>();
+        case BB_TYPE_UINT32: return _Max<uint32_t>();
+        case BB_TYPE_UINT64: return _Max<uint64_t>();
+        default:   BB_ASSERT(0);
+        }
+        return 0;
+    }
     
     FrameBuffer Sqrt(void)
     {
