@@ -38,7 +38,7 @@ public:   // debug
 
     bool                    m_binary_mode = false;
     bool                    m_host_only   = false;
-    bool                    m_host_simd   = true;
+    bool                    m_host_simd   = false;
     
     std::string             m_connection;
 
@@ -382,21 +382,53 @@ public:
         this->InitializeNodeInput(m_mt(), m_connection);
 
         // パラメータ初期化
+        m_W0->Resize({m_output_node_size, M, N}, DataType<T>::type);
+        m_b0->Resize({m_output_node_size, M},    DataType<T>::type);
+        m_W1->Resize({m_output_node_size, M},    DataType<T>::type);
+        m_b1->Resize({m_output_node_size},       DataType<T>::type);
+        
+        m_dW0->Resize({m_output_node_size, M, N}, DataType<T>::type);
+        m_db0->Resize({m_output_node_size, M},    DataType<T>::type);
+        m_dW1->Resize({m_output_node_size, M},    DataType<T>::type);
+        m_db1->Resize({m_output_node_size},       DataType<T>::type);
         if (m_initializer == "he" || m_initializer == "He") {
-            m_initialize_std = (T)2.0 / std::sqrt((T)N);
+            m_initialize_std = (T)std::sqrt((double)2.0 / (double)N);
+            m_W0->InitNormalDistribution(0.0, m_initialize_std, m_mt());
+            m_b0->InitNormalDistribution(0.0, m_initialize_std, m_mt());
+            m_W1->InitNormalDistribution(0.0, m_initialize_std, m_mt());
+            m_b1->InitNormalDistribution(0.0, m_initialize_std, m_mt());
         }
         else if (m_initializer == "xavier" || m_initializer == "Xavier" ) {
-            m_initialize_std = (T)1.0 / std::sqrt((T)N);
+            m_initialize_std = (T)std::sqrt((double)1.0 / (double)N);
+            m_W0->InitNormalDistribution(0.0, m_initialize_std, m_mt());
+            m_b0->InitNormalDistribution(0.0, m_initialize_std, m_mt());
+            m_W1->InitNormalDistribution(0.0, m_initialize_std, m_mt());
+            m_b1->InitNormalDistribution(0.0, m_initialize_std, m_mt());
         }
-        m_W0->Resize({m_output_node_size, M, N}, DataType<T>::type);    m_W0->InitNormalDistribution(0.0, m_initialize_std, m_mt());
-        m_b0->Resize({m_output_node_size, M},    DataType<T>::type);    m_b0->InitNormalDistribution(0.0, m_initialize_std, m_mt());
-        m_W1->Resize({m_output_node_size, M},    DataType<T>::type);    m_W1->InitNormalDistribution(0.0, m_initialize_std, m_mt());
-        m_b1->Resize({m_output_node_size},       DataType<T>::type);    m_b1->InitNormalDistribution(0.0, m_initialize_std, m_mt());
-
-        m_dW0->Resize({m_output_node_size, M, N}, DataType<T>::type);   m_dW0->FillZero();
-        m_db0->Resize({m_output_node_size, M},    DataType<T>::type);   m_db0->FillZero();
-        m_dW1->Resize({m_output_node_size, M},    DataType<T>::type);   m_dW1->FillZero();
-        m_db1->Resize({m_output_node_size},       DataType<T>::type);   m_db1->FillZero();
+        else if (m_initializer == "normal" || m_initializer == "Normal" ) {
+            m_W0->InitNormalDistribution(0.0, m_initialize_std, m_mt());
+            m_b0->InitNormalDistribution(0.0, m_initialize_std, m_mt());
+            m_W1->InitNormalDistribution(0.0, m_initialize_std, m_mt());
+            m_b1->InitNormalDistribution(0.0, m_initialize_std, m_mt());
+        }
+        else if (m_initializer == "uniform" || m_initializer == "Uniform" ) {
+            double k = (double)m_initialize_std * std::sqrt(3.0);
+            m_W0->InitUniformDistribution(-k, +k, m_mt());
+            m_b0->InitUniformDistribution(-k, +k, m_mt());
+            m_W1->InitUniformDistribution(-k, +k, m_mt());
+            m_b1->InitUniformDistribution(-k, +k, m_mt());
+        }
+        else {
+            double k = std::sqrt(1.0 / (double)N);
+            m_W0->InitUniformDistribution(-k, +k, m_mt());
+            m_b0->InitUniformDistribution(-k, +k, m_mt());
+            m_W1->InitUniformDistribution(-k, +k, m_mt());
+            m_b1->InitUniformDistribution(-k, +k, m_mt());
+        }
+        m_dW0->FillZero();
+        m_db0->FillZero();
+        m_dW1->FillZero();
+        m_db1->FillZero();
 
         return m_output_shape;
     }
@@ -472,28 +504,28 @@ public:
 
         // affine0
         std::vector<T> value0(M);
-        for (index_t i = 0; i < M; ++i) {
+        for (std::size_t i = 0; i < M; ++i) {
             value0[i] = b0(node, i);
-            for (index_t j = 0; j < N; ++j) {
+            for (std::size_t j = 0; j < N; ++j) {
                 value0[i] += (T)input_value[j] * W0(node, i, j);
             }
         }
 
         // ReLU
-        for (index_t i = 0; i < M; ++i) {
+        for (std::size_t i = 0; i < M; ++i) {
             value0[i] = std::max(value0[i], (T)0.0);
         }
 
         // affine1
         std::vector<T> value1(1);
         value1[0] = b1(node);
-        for (index_t i = 0; i < M; ++i) {
+        for (std::size_t i = 0; i < M; ++i) {
             value1[0] = value1[0] + value0[i] * W1(node, i);
         }
 
         // 型変換
         std::vector<double> value2(M);
-        for (index_t i = 0; i < M; ++i) {
+        for (std::size_t i = 0; i < M; ++i) {
             value2[i] = (double)value1[i];
         }
 
@@ -673,7 +705,7 @@ public:
                 for (index_t frame = 0; frame < frame_size; ++frame ) {
                     T   in_sig[N];
                     for ( int i = 0; i < N; ++i) {
-                        in_sig[i] = x_ptr.Get(frame, in_idx[i]);
+                        in_sig[i] = (T)x_ptr.Get(frame, in_idx[i]);
                     }
 
                     T   sum1 = b1_ptr(node);
