@@ -123,8 +123,17 @@ public:
         m_v = 0;
     }
     
+    void ZeroGrad(void) override
+    {
+        if ( m_params.IsEmpty() ) {
+            return;
+        }
 
-    void Update(void)
+        m_grads = 0;
+    }
+
+    
+    void Step(void) override
     {
         if ( m_params.IsEmpty() ) {
             return;
@@ -134,7 +143,7 @@ public:
         if ( m_params.IsDeviceAvailable() && m_grads.IsDeviceAvailable() && m_m.IsDeviceAvailable() && m_v.IsDeviceAvailable() && Manager::IsDeviceAvailable() ) {
             // CUDA版
             auto lr_t = m_learning_rate * std::sqrt((T)1.0 - m_b2) / ((T)1.0 - m_b1 );
-            bbcu_fp32_Adam
+            bbcu_fp32_OptimizerAdam
                     (
                         (int            )m_params.GetSize(),
                         (int     const *)m_params.GetDeviceSizeTable(),
@@ -161,7 +170,7 @@ public:
             m_v += ((T)1.0 - m_beta2) * (m_grads * m_grads - m_v);
 
             m_params -= lr_t * m_m / (Sqrt(m_v) + (T)1e-7);
-            m_grads   = 0;
+//          m_grads   = 0;
 
             m_b1 *= m_beta1;
             m_b2 *= m_beta2;
